==> mp3plot-0.5.1.tar.gz: mp3plot-0.5.1/src/bitrate_dist.h <==
/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: bitrate_dist.h 1639 2009-04-14 02:21:18Z  $

#ifndef MP3PLOT_BITRATE_DIST_H
#define MP3PLOT_BITRATE_DIST_H

#include <map>

#include "common.h"

namespace mp3plot {

    /*!
     * \brief Holds the bitrate distribution of a file
     * The bitrate distribution is a table that maps a given nominal bitrate
     * to the number of frames that use this given bitrate.
     * \note It is only meant to be filled once. See fill_from()'s precondition
     * \note Free-form bitrates aren't currently handled
     * \see fill_from()
     */
    class bitrate_distribution {
        private:
            //! Maps bitrates to the number of frames using them
            typedef std::map<bitrate_t, numframes_t> bitrate_map_t;
            //! STL Iterator to bitrate_map_t
            typedef bitrate_map_t::const_iterator    bitrate_map_base_iterator_t;

            /*!
             * \brief Wrapper around the real iterator.
             * This class wraps an iterator to abstract access to the
             * exposed members of the underlying contained elements, by using
             * bitrate() and numframes().
             * \todo XXX: Is this OK enough to work right?
             * \note This is a constant iterator
             * \note Made this way to hide completely implementation details
             * \see bitrate()
             * \see numframes()
             */
            class bitrate_table_iterator : public bitrate_map_base_iterator_t {
            protected:
                // Only bitrate_distribution can manipulate this class freely
                friend class bitrate_distribution;

                //! Copy constructor over the base iterator
                bitrate_table_iterator(std::map<bitrate_t, numframes_t>::const_iterator i)
                    : bitrate_map_base_iterator_t(i)
                {}
            public:
                //! Returns the bitrate of the pointed element
                bitrate_t bitrate() const { return (*this)->first; }
                //! Returns the number of frames of the pointed element
                numframes_t numframes() const { return (*this)->second; }
                // Prefix operator++
                //! Advances the iterator
                void operator++() { ++(*dynamic_cast<bitrate_map_base_iterator_t *>(this)); }
                // Postfix operator++
                //! Advances the iterator
                void operator++(int) { (*dynamic_cast<bitrate_map_base_iterator_t *>(this))++; }
            };

            //! The actual bitrate distribution table
            bitrate_map_t table;

        public:
            typedef bitrate_table_iterator const_iterator;

            //! The maximum number of frames with the same bitrate
            numframes_t max;
            /*!
             * \brief true if a file has already been processed
             */
            bool is_filled;

            bitrate_distribution();

            /*!
             * \brief Fills the table with statatistics from a given
             * vector of frames.
             * \param frames The frames of a file
             * \param is_vbr true if the file is a vbr file
             * \pre ! is_filled
             */
            void fill_from(const frame_collection & frames, bool is_vbr);

            // Implementation of the underlying container is hidden to
            // ease changes and abstraction

            /*!
             * \brief Returns a constant iterator to the first valid bitrate
             * \pre is_filled
             * \see bad_frames()
             * \see end()
             * \see const_iterator for details on the exposed elements
             */
            const_iterator begin() const;

            /*!
             * \brief Returns a constant iterator to the bitrate distribution end
             * \pre is_filled
             * \see begin()
             */
            const_iterator end() const { return bitrate_table_iterator( table.end() ); }

            /*!
             * \brief Returns the number of frames with an invalid bitrate
             * \pre is_filled
             */
            numframes_t bad_frames() const;
    };

} // namespace mp3plot

#endif // MP3PLOT_BITRATE_DIST_H
// vim:set ts=4 et ai:

==> mp3plot-0.6.0.tar.gz: mp3plot-0.6.0/src/bitrate_dist.h <==
/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: bitrate_dist.h 1342 2009-06-21 23:41:45Z toni $

#ifndef MP3PLOT_BITRATE_DIST_H
#define MP3PLOT_BITRATE_DIST_H

#include <map>

#include "common.h"

namespace mp3plot {

    /*!
     * \brief Holds the bitrate distribution of a file
     * The bitrate distribution is a table that maps a given nominal bitrate
     * to the number of frames that use this given bitrate.
     * \note It is only meant to be filled once. See fill_from()'s precondition
     * \note Free-form bitrates aren't currently handled
     * \see fill_from()
     */
    class bitrate_distribution {
#if 0
        private:
            //! Maps bitrates to the number of frames using them
            typedef std::map<bitrate_t, numframes_t> bitrate_map_t;
            //! STL Iterator to bitrate_map_t
            typedef bitrate_map_t::const_iterator    bitrate_map_base_iterator_t;

            /*!
             * \brief Wrapper around the real iterator.
             * This class wraps an iterator to abstract access to the
             * exposed members of the underlying contained elements, by using
             * bitrate() and numframes().
             * \todo XXX: Is this OK enough to work right?
             * \note This is a constant iterator
             * \note Made this way to hide completely implementation details
             * \see bitrate()
             * \see numframes()
             */
            class bitrate_table_iterator : public bitrate_map_base_iterator_t {
            protected:
                // Only bitrate_distribution can manipulate this class freely
                friend class bitrate_distribution;

                //! Copy constructor over the base iterator
                bitrate_table_iterator(std::map<bitrate_t, numframes_t>::const_iterator i)
                    : bitrate_map_base_iterator_t(i)
                {}
            public:
                //! Returns the bitrate of the pointed element
                bitrate_t bitrate() const { return (*this)->first; }
                //! Returns the number of frames of the pointed element
                numframes_t numframes() const { return (*this)->second; }
                // Prefix operator++
                //! Advances the iterator
                void operator++() { ++(*dynamic_cast<bitrate_map_base_iterator_t *>(this)); }
                // Postfix operator++
                //! Advances the iterator
                void operator++(int) { (*dynamic_cast<bitrate_map_base_iterator_t *>(this))++; }
            };
#endif
        private:
            //! Maps bitrate indexes to the number of frames using them
            typedef std::map<uint8_t, numframes_t> bitrate_map_t;
            //! STL Iterator to bitrate_map_t
            typedef bitrate_map_t::const_iterator    bitrate_map_base_iterator_t;

            /*!
             * \brief Wrapper around the real iterator.
             * This class wraps an iterator to abstract access to the
             * exposed members of the underlying contained elements, by using
             * bitrate() and numframes().
             * \todo XXX: Is this OK enough to work right?
             * \note This is a constant iterator
             * \note Made this way to hide completely implementation details
             * \see bitrate()
             * \see numframes()
             */
            class bitrate_table_iterator : public bitrate_map_base_iterator_t {
            protected:
                // Only bitrate_distribution can manipulate this class freely
                friend class bitrate_distribution;

                //! Copy constructor over the base iterator
                bitrate_table_iterator(std::map<uint8_t, numframes_t>::const_iterator i)
                    : bitrate_map_base_iterator_t(i)
                {}
            public:
                //! Returns the bitrate index of the pointed element
                uint8_t bitrate_index() const { return (*this)->first; }
                //! Returns the number of frames of the pointed element
                numframes_t numframes() const { return (*this)->second; }
                // Prefix operator++
                //! Advances the iterator
                void operator++() { ++(*dynamic_cast<bitrate_map_base_iterator_t *>(this)); }
                // Postfix operator++
                //! Advances the iterator
                void operator++(int) { (*dynamic_cast<bitrate_map_base_iterator_t *>(this))++; }
            };

            //! The actual bitrate distribution table
            bitrate_map_t table;
        public:
            typedef bitrate_table_iterator const_iterator;

            //! The maximum number of frames with the same bitrate
            numframes_t max;
            /*!
             * \brief true if a file has already been processed
             */
            bool is_filled;

            bitrate_distribution();

            /*!
             * \brief Fills the table with statatistics from a given
             * vector of frames.
             * \param frames The frames of a file
             * \param is_vbr true if the file is a vbr file
             * \pre ! is_filled
             */
            void fill_from(const frame_collection & frames, bool is_vbr);

            // Implementation of the underlying container is hidden to
            // ease changes and abstraction

            /*!
             * \brief Returns a constant iterator to the first valid bitrate
             * \pre is_filled
             * \see bad_frames()
             * \see end()
             * \see const_iterator for details on the exposed elements
             */
            const_iterator begin() const;

            /*!
             * \brief Returns a constant iterator to the bitrate distribution end
             * \pre is_filled
             * \see begin()
             */
            const_iterator end() const {
                // The bad bitrate value is now at the end
                bitrate_map_base_iterator_t it = table.end();
                --it;
                assert( it->first == 0x0F );
                return bitrate_table_iterator( it );
                //return bitrate_table_iterator( table.end() );
            }

            /*!
             * \brief Returns the number of frames with an invalid bitrate
             * \pre is_filled
             */
            numframes_t bad_frames() const;

            /*!
             * \brief Returns the number of different bitrates in the distribution
             * \remarks This isn't necessarily the same as the standard number of
             *          bitrates (16), e.g. on concatenated files or files switching
             *          mpeg versions from frame to frame the total number of bitrates
             *          might be different
             */
            uint8_t bitrates_number() const {
                assert( is_filled );
                return table.size();
            }
    };

} // namespace mp3plot

#endif // MP3PLOT_BITRATE_DIST_H
// vim:set ts=4 et ai:
